;;
;; climber.el - a man climbs a window
;;   by Hirotsugu Kakugawa
;; 


(setq climber-pattern-1  
      '(" ._  "
        "<(_) "
	" [_]>"
	" < > "))
(setq climber-pattern-2 
      '("  _ ."
        " (_)>"
	"<[_] "
	" < > "))
(setq climber-pattern-3 
      '(" ._ ."
        "<(_)>"
	" [_] "
	" / \\ "))
(setq climber-pattern-4 
      '("  _  "
        " (_) "
	"<[_]>"
	" < > "))

(defun climber ()
  (interactive)
  (let ((buff (get-buffer-create "*climber*"))
	(winh (window-height))
	(winw (window-width))
	(xpos 0))
    (switch-to-buffer buff)
    (while t
      (setq xpos (random (- winw 15)))
      (climber-buffer winw winh xpos)
      (climber-climb winw winh xpos))))

(defun climber-buffer (winw winh xpos)
  (erase-buffer)
  (let ((s (make-string (- winw 2) ?\ ))
	(h (+ winh (length climber-pattern-1))))
    (while (> h 0)
      (insert s)
      (insert "\n")
      (setq h (- h 1))))
  (goto-char (point-min))
  (goto-char xpos)
  (let ((h (+ winh (length climber-pattern-1))))
    (while (> h 0)
      (climber-draw-ladder h xpos)
      (next-line 1)
      (setq h (- h 1))))
  (goto-char (point-min)))

(defun climber-draw-ladder (y xpos)
  (save-excursion
    (beginning-of-line)
    (forward-char xpos)
    (delete-char (string-width "|+-----+|") nil)
    (if (= (mod y 2) 0)
	(insert "|+-----+|")
      (insert "|+-----+|"))))



(defun climber-climb (winw winh x)
  (let ((y winh)
	tt patt i ii)
    (setq tt 0)
    (while (> y (- 0 (length climber-pattern-1) 2))
      (if (and (> tt 0) 
	       (= (mod tt 6) 0)
	       (= (random 4) 0))
	  (progn
	    (climber-draw x (+ y 1) (climber-pattern (- tt 1))
			  (+ (random 2) 1))
	    (setq i (+ 1 (random 4)))
	    (setq ii 1)
	    (while (> i 0)
	      (setq y (+ y 1))
	      (climber-draw x y climber-pattern-3 (+ ii (random 3)))
	      (setq y (- y 1))
	      (climber-draw x y climber-pattern-4 (+ 1 (random 3)))
	      (setq i (- i 1))
	      (setq ii (+ ii 2)))
	    (setq y (+ y 1)))
	(progn
	  (climber-draw x y (climber-pattern tt) 1)))
      (setq tt (+ tt 1))
      (setq y (- y 1)))))

(defun climber-pattern (tt)
  (if (= (mod tt 2) 0) 
      climber-pattern-1
    climber-pattern-2))


(defun climber-draw (x y patt rest)
  (climber-draw* x y patt t)
  (sit-for rest)
  (climber-draw* x y patt nil))

(defun climber-draw* (x y patt draw)
  (let* ((si)
	 (opoint))
    (goto-char (point-min))
    (save-excursion
      (if (>= y 1)
	  (forward-line (- y 1)))
      (forward-char (+ x 2))
      (setq si 0)
      (while (not (null patt))
	(if (>= (+ y si) 1)
	    (progn
	      (setq opoint (point))
	      (if draw
		  (progn 
		    (delete-char (string-width (car patt)) nil)		
		    (insert (car patt)))
		(progn
		  ;;(delete-char (string-width (car patt)) nil)
		  ;;(insert (make-string (string-width (car patt)) ?\ ))
		  (climber-draw-ladder y x)))
	      (goto-char opoint)
	      (next-line 1)))
	(setq patt (cdr patt))
	(setq si (+ si 1)) ))))


;; end
